/*
 * tinyTimeTrailer.c
 *
 * Created: 04.12.2016 22:34:13
 * Author : Chris
 */ 

#include <avr/io.h>
#include <avr/sleep.h>
#include <avr/interrupt.h>
#include <avr/wdt.h>

#define TIMEOUT 30
//#define ADC_DEBUG

//calculation: ADC_THRESHOLD = Ubat * (10k / (33k + 10k)) * 1024 / 1.1 V
//here: 3.3 V
//#define ADC_THRESHOLD 714

#define ADC_THRESHOLD_mV 3300
#define ADC_THRESHOLD1 (uint16_t)(ADC_THRESHOLD_mV * 10240UL / 47300UL)
#define ADC_THRESHOLD ( ADC_THRESHOLD1 )
#define nop() asm volatile("nop")

ISR(WDT_vect)
{
	//do nothing, just wake up again
}

ISR(ADC_vect)
{
	//do nothing, just wake up again
}

uint16_t readAdc()
{
	//ref int 1.1 V, data right adjust, source ADC2 (PB4)
	ADMUX = (1<<REFS0) | (0<<ADLAR) | (1<<MUX1) | (0<<MUX0);
	//ADC enable, start conversion, enable interrupt, clock divider 8 => 75 kHz
	ADCSRA = (1<<ADEN) | (1<<ADSC) | (1<<ADIE) | (0<<ADPS2) | (1<<ADPS2) | (1<<ADPS2);
	
	//sleep while conversion
	set_sleep_mode(SLEEP_MODE_ADC);
	sleep_mode();
	
	uint16_t retval = ADC;
	
	ADCSRA = (0<<ADEN) | (0<<ADSC) | (0<<ADIE);
	return retval;
}

//write a 10 bit value with approx. 49000 baud 10N1 to PB1
void dbg_write10(uint16_t val)
{
	PORTB &= ~(1<<PB1);
	nop(); nop();
	nop(); nop();
	nop(); nop();
	nop(); 	
	
	for(uint8_t i = 0; i < 10; i++)
	{
		if(val & 1)
		{
			PORTB |= (1<<PB1);
		}
		else
		{
			PORTB &= ~(1<<PB1);
			nop(); nop();
		}
		
		val >>= 1;
	}
	
	PORTB |= (1<<PB1);
}

int main(void)
{
	//configure PB3 as output and turn it on (turn on LEDs)
	DDRB |= (1<<PB3);
	PORTB |= (1<<PB3);

#ifdef ADC_DEBUG
	DDRB |= (1<<PB1);
	PORTB |= (1<<PB1);
#endif
	
	//configure watchdog to cause an interrupt every second
	WDTCR = (1<<WDCE) | (1<<WDE);
	WDTCR = (0<<WDCE) | (0<<WDE) | (1<<WDTIE) | WDTO_1S;

	uint8_t cnt = 0;
	
	//enable interrupts (for ADC and watchdog)
	sei();
	
	
    while (1) 
    {
		cnt++;
		PORTB |= (1<<PB3);
		
#ifdef ADC_DEBUG
		uint16_t tmp = readAdc();
		dbg_write10(tmp);
#endif
		
		if(cnt == 2)
		{
			//at beginning of second 2, measure ADC and turn off LEDs for the rest of the cycle
			uint16_t adcval = readAdc();
			
			if(adcval < ADC_THRESHOLD)
			{
				PORTB &= ~(1<<PB3);
			}
		}
		
		if(cnt >= TIMEOUT)
		{
			//we're done. Turn off the light and disable the watchdog.
			//This will lead to a full shutdown with the next sleep_mode
			PORTB &= ~(1<<PB3);
			wdt_disable();
		}
		
		set_sleep_mode(SLEEP_MODE_PWR_DOWN);
		sleep_mode();
    }
}

